/*
* JBoss, Home of Professional Open Source
* Copyright 2007, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package sample.vpd.servlet;

import java.io.IOException;

import javax.ejb.EJBException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jboss.logging.Logger;

import sample.vpd.ejb.BankLocal;

public class ControllerServlet
   extends HttpServlet
{
   /** The serialVersionUID */
   private static final long serialVersionUID = -2563340610848943348L;

   // TODO: inject this resource
   BankLocal bankLocal;
   
   private Logger log = Logger.getLogger(ControllerServlet.class);
   
   public void doGet(HttpServletRequest request,
                     HttpServletResponse response)
     throws IOException, ServletException
   {
      processRequest(request, response);
   }
   
   public void doPost(HttpServletRequest request,
                     HttpServletResponse response)
      throws IOException, ServletException
   {
      processRequest(request, response);
   }

   public void processRequest(HttpServletRequest request,
                              HttpServletResponse response)
      throws IOException, ServletException
   {
     log.info("processRequest");
     String actionName = request.getParameter("action");
     String accountName = request.getParameter("accountName");
     Double newBalance = null;
     Double currentBalance = null;
     String newBalanceString = request.getParameter("balance");
     
     if (newBalanceString != null)
     {
        // TODO: check for format errors here or earlier
        newBalance = Double.valueOf(newBalanceString);
     }
     
     log.info("User "+getUser(request)+" called processRequest[action="+actionName+", account="+accountName+", balance="+newBalance+"]");
     
     String destinationPage = "/error.jsp";
     
     if (actionName.equals("getBalance"))
     {
        currentBalance = getBankLocal().getBalance(accountName);
        log.info("Bank.getCurrentBalance() returned "+currentBalance);
        request.setAttribute("balance", currentBalance);
        request.setAttribute("accountName", accountName);
        request.setAttribute("user", getUser(request));
        destinationPage = "/getBalance.jsp";
     }
     else if (actionName.equals("setBalance"))
     {
        try
        {
           getBankLocal().setBalance(accountName, newBalance);
           request.setAttribute("balance", newBalance);
           request.setAttribute("accountName", accountName);
           request.setAttribute("user", getUser(request));
           destinationPage = "/setBalance.jsp";
        }
        catch (EJBException ex)
        {
           if (ex.getCausedByException() instanceof SecurityException)
           {
              SecurityException sex = (SecurityException) ex.getCausedByException();
              
              request.setAttribute("accountName", accountName);
              request.setAttribute("user", getUser(request));
              request.setAttribute("securityException", sex);
              request.setAttribute("securityExceptionDetails", sex.getMessage());
              destinationPage = "/setBalance.jsp";
           }
           else
           {
              throw(ex);
           }
        }
     }
     else if (actionName.equals("index"))
     {
        request.setAttribute("user", getUser(request));
        destinationPage = "/index.jsp";
     }
     else if (actionName.equals("logout"))
     {
        request.getSession().invalidate();
        destinationPage = "/index.jsp";
     }
     else
     {
        destinationPage = "/error.jsp";
     }
     
     getServletContext().getRequestDispatcher(destinationPage).forward(request, response);
   }
   
   //TODO: replace with resource injection
   private BankLocal getBankLocal()
   {
      if (bankLocal == null)
      {
         try
         {
            Context ctx = new InitialContext();
            
            bankLocal = (BankLocal) ctx.lookup("/OracleVPD/Bank/local");
         }
         catch (NamingException e)
         {
            // FIXME
            log.error("Error", e);
         }
      }
      
      return bankLocal;
   }
   
   private String getUser(HttpServletRequest request)
   {
      return request.getUserPrincipal().getName();
   }
}
